External Entities Database plugin
=================================

CONTENTS OF THIS FILE
---------------------

 * Introduction
 * Requirements
 * Installation
 * Configuration
 * Troubleshooting
 * FAQ
 * Maintainers

INTRODUCTION
------------

The Database Cross-Schema Query API extension provides interfaces, traits,
plugin support and other features to enable querying multiple database schema
in a single SQL query. This API is based on Drupal database API and extends it
to support multiple schema specification.

For instance, if your DBMS holds other schemas beside Drupal's schema, it
becomes possible to query those schemas in a single SQL query using Drupal
database API. Let's say you have a table "sometable" in a schema "myfirstschema"
and another table "othertable" in another schema "myotherschema", and those
tables contain data related to Drupal node table, you could use this kind of
query to cross data:
```
SELECT *
FROM public.node n
  JOIN myfirstschema.sometable t1 ON t1.node_identifier = n.nid
  LEFT JOIN myotherschema.othertable t2 ON t2.node_identifier = n.nid
    AND t2.some_field = t1.some_field
WHERE
  t1.title LIKE 'Something%'
```
Such a query would be run using the Database Cross-Schema API the following way:
```
// Get a cross-connection object on first schema.
$xconnection =
  \Drupal\dbxschema\Database\DatabaseTool::getConnection('myfirstschema');
// Add second schema.
$xconnection->addExtraSchema('myotherschema');
$result = $xconnection->query('
  SELECT *
  FROM {node} n
    JOIN {1:sometable} t1 ON t1.node_identifier = n.nid
    LEFT JOIN {2:othertable} t2 ON t2.node_identifier = n.nid
      AND t2.some_field = t1.some_field
  WHERE
    t1.title LIKE :title;',
  [':title' => 'Something%']
);
```
This is just a silly example to demonstrate how to use the API.

This API is built on Drupal database API, which means you can use a
cross-connection object exactly the same way you would use a Drupal connection
object. But it supports an additional syntax for tables that allows to select
the schema they belong to: while you use curly braces arround table names for
Drupal tables, you can now prefix table names with a number corresponding to the
schema you want. By default, you will instanciate a cross-connection object with
a schema name and then, you are free to add as many schema names you want to the
connection using the ::addExtraSchema() method. Many other methods are available
to manage schemas (see DatabaseTool, CrossSchemaConnectionInterface and
CrossSchemaSchemaInterface documentation).

By default, this API is database agnostic and does not work on its own. It
requires plugins that implement database drivers. This extension provides 2
plugin modules: one for the PostgreSQL driver (dbxschema_pgsql) and one for the
MySQL driver (dbxschema_mysql). Note that, in the case of MySQL, schemas are
actual MySQL databases accessible on a same MySQL server through a same user
account.

 * For a full description of the module, visit the project page:
   https://www.drupal.org/project/dbxschema

 * To submit bug reports and feature suggestions, or track changes:
   https://www.drupal.org/project/issues/dbxschema

REQUIREMENTS
------------

This module requires the following modules:

 * [External Entities](https://www.drupal.org/project/external_entities)

INSTALLATION
------------

 * Install as you would normally install a contributed Drupal module. Visit
   https://www.drupal.org/node/1897420 for further information.

CONFIGURATION
-------------

The module has no menu or modifiable settings. There is no configuration. When
enabled, the module will add a new storage client for external entity types.

TROUBLESHOOTING
---------------

 * If:

   - Are ?

   - Does ?

FAQ
---

Q: I ?

A: Yes.

MAINTAINERS
-----------

Current maintainers:
 * Valentin Guignon (guignonv) - https://www.drupal.org/u/guignonv
